@@ -11,8 +11,8 @@
  * On MMU machine, the transform_prog is removed by macro magic
  * in include/unarchive.h. On NOMMU, transformer is removed.
  */
-int open_transformer(int src_fd,
-	USE_DESKTOP(long long) int (*transformer)(int src_fd, int dst_fd),
+void FAST_FUNC open_transformer(int fd,
+	USE_DESKTOP(long long) int FAST_FUNC (*transformer)(int src_fd, int dst_fd),
 	const char *transform_prog)
 {
 	struct fd_pair fd_pipe;
@@ -23,28 +23,29 @@ int open_transformer(int src_fd,
 #if BB_MMU
 	pid = fork();
 	if (pid == -1)
-		bb_perror_msg_and_die("can't fork");
+		bb_perror_msg_and_die("vfork" + 1);
 #else
 	pid = vfork();
 	if (pid == -1)
-		bb_perror_msg_and_die("can't vfork");
+		bb_perror_msg_and_die("vfork");
 #endif
 
 	if (pid == 0) {
 		/* child process */
-		close(fd_pipe.rd); /* We don't want to read from the parent */
+		close(fd_pipe.rd); /* we don't want to read from the parent */
 		// FIXME: error check?
 #if BB_MMU
-		transformer(src_fd, fd_pipe.wr);
+		transformer(fd, fd_pipe.wr);
 		if (ENABLE_FEATURE_CLEAN_UP) {
-			close(fd_pipe.wr); /* Send EOF */
-			close(src_fd);
+			close(fd_pipe.wr); /* send EOF */
+			close(fd);
 		}
-		exit(EXIT_SUCCESS);
+		/* must be _exit! bug was actually seen here */
+		_exit(EXIT_SUCCESS);
 #else
 		{
 			char *argv[4];
-			xmove_fd(src_fd, 0);
+			xmove_fd(fd, 0);
 			xmove_fd(fd_pipe.wr, 1);
 			argv[0] = (char*)transform_prog;
 			argv[1] = (char*)"-cf";
@@ -58,7 +59,6 @@ int open_transformer(int src_fd,
 	}
 
 	/* parent process */
-	close(fd_pipe.wr); /* Don't want to write to the child */
-
-	return fd_pipe.rd;
+	close(fd_pipe.wr); /* don't want to write to the child */
+	xmove_fd(fd_pipe.rd, fd);
 }
