@@ -39,7 +39,7 @@ static const char env_longopts[] ALIGN1 =
 #endif
 
 int env_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int env_main(int argc ATTRIBUTE_UNUSED, char **argv)
+int env_main(int argc UNUSED_PARAM, char **argv)
 {
 	/* cleanenv was static - why? */
 	char *cleanenv[1];
