@@ -15,7 +15,7 @@
 /* This is a NOFORK applet. Be very careful! */
 
 int false_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int false_main(int ATTRIBUTE_UNUSED argc, char ATTRIBUTE_UNUSED **argv)
+int false_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 {
 	return EXIT_FAILURE;
 }
