@@ -24,27 +24,33 @@ int mknod_main(int argc, char **argv)
 	dev_t dev;
 	const char *name;
 
-	mode = getopt_mk_fifo_nod(argc, argv);
+	mode = getopt_mk_fifo_nod(argv);
 	argv += optind;
 	argc -= optind;
 
-	if ((argc >= 2) && ((name = strchr(modes_chars, argv[1][0])) != NULL)) {
-		mode |= modes_cubp[(int)(name[4])];
+	if (argc >= 2) {
+		name = strchr(modes_chars, argv[1][0]);
+		if (name != NULL) {
+			mode |= modes_cubp[(int)(name[4])];
 
-		dev = 0;
-		if ((*name != 'p') && ((argc -= 2) == 2)) {
-			/* Autodetect what the system supports; these macros should
-			 * optimize out to two constants. */
-			dev = makedev(xatoul_range(argv[2], 0, major(UINT_MAX)),
-			              xatoul_range(argv[3], 0, minor(UINT_MAX)));
-		}
+			dev = 0;
+			if (*name != 'p') {
+				argc -= 2;
+				if (argc == 2) {
+					/* Autodetect what the system supports; these macros should
+					 * optimize out to two constants. */
+					dev = makedev(xatoul_range(argv[2], 0, major(UINT_MAX)),
+					              xatoul_range(argv[3], 0, minor(UINT_MAX)));
+				}
+			}
 
-		if (argc == 2) {
-			name = *argv;
-			if (mknod(name, mode, dev) == 0) {
-				return EXIT_SUCCESS;
+			if (argc == 2) {
+				name = *argv;
+				if (mknod(name, mode, dev) == 0) {
+					return EXIT_SUCCESS;
+				}
+				bb_simple_perror_msg_and_die(name);
 			}
-			bb_simple_perror_msg_and_die(name);
 		}
 	}
 	bb_show_usage();
