@@ -14,7 +14,7 @@
 /* This is a NOFORK applet. Be very careful! */
 
 int whoami_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int whoami_main(int argc, char **argv ATTRIBUTE_UNUSED)
+int whoami_main(int argc, char **argv UNUSED_PARAM)
 {
 	if (argc > 1)
 		bb_show_usage();
