@@ -4,7 +4,8 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 #include "libbb.h"
-char* str_tolower(char *str)
+
+char* FAST_FUNC str_tolower(char *str)
 {
 	char *c;
 	for (c = str; *c; ++c)
