@@ -32,7 +32,7 @@ done
  */
 
 int cryptpw_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int cryptpw_main(int argc ATTRIBUTE_UNUSED, char **argv)
+int cryptpw_main(int argc UNUSED_PARAM, char **argv)
 {
 	char salt[sizeof("$N$XXXXXXXX")];
 	char *opt_a;
