@@ -85,7 +85,7 @@ static int open_as_user(const struct passwd *pas, const char *file)
 }
 
 int crontab_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int crontab_main(int argc ATTRIBUTE_UNUSED, char **argv)
+int crontab_main(int argc UNUSED_PARAM, char **argv)
 {
 	const struct passwd *pas;
 	const char *crontab_dir = CRONTABS;
@@ -93,7 +93,6 @@ int crontab_main(int argc ATTRIBUTE_UNUSED, char **argv)
 	char *new_fname;
 	char *user_name;  /* -u USER */
 	int fd;
-	int src_fd;
 	int opt_ler;
 
 	/* file [opts]     Replace crontab from file
@@ -130,11 +129,11 @@ int crontab_main(int argc ATTRIBUTE_UNUSED, char **argv)
 		if (!pas)
 			bb_error_msg_and_die("user %s is not known", user_name);
 	} else {
+/* XXX: xgetpwuid */
 		uid_t my_uid = getuid();
 		pas = getpwuid(my_uid);
 		if (!pas)
-			bb_perror_msg_and_die("no user record for UID %u",
-					(unsigned)my_uid);
+			bb_perror_msg_and_die("unknown uid %d", (int)my_uid);
 	}
 
 #define user_name DONT_USE_ME_BEYOND_THIS_POINT
@@ -145,15 +144,15 @@ int crontab_main(int argc ATTRIBUTE_UNUSED, char **argv)
 		bb_show_usage();
 
 	/* Read replacement file under user's UID/GID/group vector */
-	src_fd = STDIN_FILENO;
 	if (!opt_ler) { /* Replace? */
 		if (!argv[0])
 			bb_show_usage();
 		if (NOT_LONE_DASH(argv[0])) {
-			src_fd = open_as_user(pas, argv[0]);
-			if (src_fd < 0)
+			fd = open_as_user(pas, argv[0]);
+			if (fd < 0)
 				bb_error_msg_and_die("user %s cannot read %s",
 						pas->pw_name, argv[0]);
+			xmove_fd(fd, STDIN_FILENO);
 		}
 	}
 
@@ -181,23 +180,23 @@ int crontab_main(int argc ATTRIBUTE_UNUSED, char **argv)
 		tmp_fname = xasprintf("%s.%u", crontab_dir, (unsigned)getpid());
 		/* No O_EXCL: we don't want to be stuck if earlier crontabs
 		 * were killed, leaving stale temp file behind */
-		src_fd = xopen3(tmp_fname, O_RDWR|O_CREAT|O_TRUNC, 0600);
-		fchown(src_fd, pas->pw_uid, pas->pw_gid);
+		fd = xopen3(tmp_fname, O_RDWR|O_CREAT|O_TRUNC, 0600);
+		xmove_fd(fd, STDIN_FILENO);
+		fchown(STDIN_FILENO, pas->pw_uid, pas->pw_gid);
 		fd = open(pas->pw_name, O_RDONLY);
 		if (fd >= 0) {
-			bb_copyfd_eof(fd, src_fd);
+			bb_copyfd_eof(fd, STDIN_FILENO);
 			close(fd);
-			xlseek(src_fd, 0, SEEK_SET);
 		}
-		close_on_exec_on(src_fd); /* don't want editor to see this fd */
 		edit_file(pas, tmp_fname);
+		xlseek(STDIN_FILENO, 0, SEEK_SET);
 		/* fall through */
 
 	case 0: /* Replace (no -l, -e, or -r were given) */
 		new_fname = xasprintf("%s.new", pas->pw_name);
 		fd = open(new_fname, O_WRONLY|O_CREAT|O_TRUNC|O_APPEND, 0600);
 		if (fd >= 0) {
-			bb_copyfd_eof(src_fd, fd);
+			bb_copyfd_eof(STDIN_FILENO, fd);
 			close(fd);
 			xrename(new_fname, pas->pw_name);
 		} else {
