@@ -179,7 +179,7 @@ static int get_wol_pw(const char *ethoptarg, unsigned char *wol_passwd)
 }
 
 int ether_wake_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int ether_wake_main(int argc ATTRIBUTE_UNUSED, char **argv)
+int ether_wake_main(int argc UNUSED_PARAM, char **argv)
 {
 	const char *ifname = "eth0";
 	char *pass;
