@@ -123,7 +123,7 @@ static uint8_t* alloc_dhcp_option(int code, const char *str, int extra)
 
 
 int udhcpc_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int udhcpc_main(int argc ATTRIBUTE_UNUSED, char **argv)
+int udhcpc_main(int argc UNUSED_PARAM, char **argv)
 {
 	uint8_t *temp, *message;
 	char *str_c, *str_V, *str_h, *str_F, *str_r;
