@@ -5,7 +5,7 @@
 #include "libbb.h"
 
 int load_policy_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int load_policy_main(int argc, char **argv ATTRIBUTE_UNUSED)
+int load_policy_main(int argc, char **argv UNUSED_PARAM)
 {
 	int rc;
 
