@@ -38,7 +38,7 @@ struct serial_struct {
 };
 
 int cttyhack_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int cttyhack_main(int argc ATTRIBUTE_UNUSED, char **argv)
+int cttyhack_main(int argc UNUSED_PARAM, char **argv)
 {
 	int fd;
 	char console[sizeof(int)*3 + 16];
